(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-printers")

(frame:on-leave
   (lambda()
     (and (group2 visibility) 'need-confirm)))

(frame:on-apply
   (lambda() (modify-printer)))


(define (get-printers)
  (woo-catch
   (lambda()
     (woo-list-names "/cupsd/printers"))
   (lambda(reason) '())))

(define (get-server)
  (woo-catch
   (lambda()
     (woo-read-first "/cupsd/server"))
   (lambda(reason) '())))


(define (printer name)
  (woo-catch
   (lambda()
      (woo-read-first (string-append "/cupsd/printers/" name)))
   (lambda(reason)
     '())))


;current printer data
(define server (get-server))
(define printer-type (make-cell #f))
(define printer-data (make-cell #f))

;edit/new data
(define printer-status (make-cell #f))
(define printer-uri (make-cell #f))
(define printer-driver (make-cell #f))

;edit/new mode
(define mode (make-cell 'edit))

(define (redraw-status-button)
  (state-button text (if (cell-ref printer-status)
                         (_ "stop printer")
                         (_ "start printer")))
  (state-button pixmap (if (cell-ref printer-status)
                           "printers/pause.png"
                           "printers/play.png")))

(define (update-status-button prn)
  (cell-set! printer-status
             (string=? (woo-get-option prn 'status) "started"))
  (redraw-status-button))

(define (extract-model prn)
  (let* ((driver (woo-get-option prn 'driver))
         (name (cond-cadr (string-cut driver #\;))))
    (or name (_ "Unknown model"))))

(define (update-connection-button mdl)
  (connection-button text
              (string-append
               (cond
                ((string=? (cell-ref printer-type) "smb") (_ "Samba"))
                ((string=? (cell-ref printer-type) "lcl") (_ "Local"))
		(else (_"Unknown connection")))
               "/"
	       (or mdl (_ "Unknown model")))))
  
(define (update-connection uri)
    (cond
     ((string-starts-with? uri "smb://") (cell-set! printer-type "smb"))
     ((string-starts-with? uri "parallel:") (cell-set! printer-type "lcl"))
     ((string-starts-with? uri "usb:") (cell-set! printer-type "lcl"))
     (else (cell-set! printer-type "sel"))))

(define (fill-info name)
  (let ((printer (printer name)))
        (cell-set! printer-data printer)
        (cell-set! printer-uri #f)
  	(printername text name)
        (update-connection (woo-get-option printer 'uri))
        (update-connection-button (extract-model printer))
  	(update-status-button printer)))

(define (mark-default name)
  (let ((default (woo-get-option server 'default)))
    (cons name
          (if (string=? name default) "theme:radio-on" "theme:radio-off"))))

(define (do-default)
  (define (extract-text x) (car (vector-ref x 0)))
  (woo-catch
   (lambda ()
     (let ((old-idx (string-list-index (woo-get-option (get-server)
                                                       'default)
                                       (map extract-text (printerlist rows))))
           (new-idx (printerlist current)))
       (woo-write "/cupsd/server" 'default (printerlist text))
       (printerlist row-item old-idx (cons #f "theme:radio-off")
                    row-item new-idx (cons #f "theme:radio-on"))))
   (lambda (reason) '())))

(define (do-test-page)
  (woo-catch
   (lambda()
     (woo-write "/cupsd/test_page" 'printer (printerlist text)))
   (lambda(reason)
     (document:popup-critical (_ "Unable to send test page") 'ok))))

(define (do-edit)
  (switch-state))

(define (do-new)
  (switch-state)
  ;new specific actions
  (cell-set! mode 'new)
  
  (printername activity #t)
  (printername text "")
  (cell-set! printer-data '())
  (cell-set! printer-driver #f)
  (cell-set! printer-uri #f)
  (cell-set! printer-type "sel")
  (connection-button text (_ "Create connection"))
  
  (cell-set! printer-status #t)
  (redraw-status-button)
  
  (printerlist append-row (cons (_ "NEW PRINTER") "theme:radio-off"))
  (printerlist current (- (printerlist count) 1)))

;TODO: add support for change type here
(define (change-connection)
  (error-message invisible)
  (let ((result (document:popup (string-append "/printers/" (cell-ref printer-type))
                                'printer-uri (or (cell-ref printer-uri)
                                                 (woo-get-option (cell-ref printer-data) 'uri))
                                'printer-driver (or (cell-ref printer-driver)
                                                    (string-cut (woo-get-option (cell-ref printer-data) 'driver) #\;)))))
    (and (pair? result)
         (begin (cell-set! printer-uri (car result))
                (cell-set! printer-driver (cdr result))
		(update-connection (cell-ref printer-uri))
		(update-connection-button (cond-cadr (cell-ref printer-driver)))))))

(define (check-buttons)
  ((widgets edit-button
           delete-button
           testpage-button
           default-button)
	   activity (> (printerlist count) 0)))

(define (switch-state)
  (error-message invisible)
  (printerlist activity (not (printerlist activity)));switch
  (let ((state (printerlist activity)))
    (group1 visibility state)
    (group2 visibility (not state))
    
    (for-each
     (lambda(w) (w activity (not state)))
     (list connection-button
           state-button))

    (for-each
     (lambda(w) (w activity state))
     (list testpage-button
           default-button)))
    (check-buttons))

(define (write-error msg)
  (error-message visibility #t)
  (error-message text (bold-red msg))
  #f)

(define (cleanup-model-name vendor model)
  (and vendor
       model
       (let* ((cutted (substring model (string-length vendor)
                                 (string-length model)))
              (start (string-skip cutted #\space)))
                   (substring cutted start (string-length cutted)))))

(define (get-driver)
  (let* ((vendor (car (cell-ref printer-driver)))
         (model (cleanup-model-name vendor (cadr (cell-ref printer-driver)))))
    (woo-get-option
     (woo-first-command
      (woo-read (string-append "/printer_drivers/" vendor "/" model)))
     'id)))

(define (modify-printer)
  (cond
   ((empty-string? (printername text))
    (write-error (_ "You should define printer name")))
   ((not (string-match "[a-z]+" (printername text)))
    (write-error (_ "Printer name contains illegal symbols (only small latin letters allowed)")))
   ((and (eq? (cell-ref mode) 'new) (not (cell-ref printer-driver)))
    (write-error (_ "You should define printer model")))
   ((and (eq? (cell-ref mode) 'new) (not (cell-ref printer-uri)))
    (write-error (_ "You shoud define printer connection")))
   (else
    (splash-message (_ "Generating printer driver..."))
    (begin-1
     (woo-catch
      (lambda()
        (document:release)
        (let ((cmd-name (string-append "/cupsd/printers/" (printername text))))
          (and (cell-ref printer-uri) (woo-write cmd-name 'uri (cell-ref printer-uri)))
          (and (cell-ref printer-driver) (woo-write cmd-name  'ppd (get-driver)))
          (woo-write cmd-name  'status (if (cell-ref printer-status) "started" "stopped")))
        #t)
      (lambda(reason) (write-error reason)))
     (splash-message "")))))


(define (do-delete)
     (and (eq? 'yes (document:popup-warning
                   (string-append (_ "Are you really want to remove printer ")
                                  (printerlist text)
                                  " ?")
                   'no
                   'yes))
          (woo-catch
           (lambda()
             (woo-delete (string-append "/cupsd/printers/" (printerlist text)))
             (printerlist remove (printerlist current))
	     (check-buttons)
	     (and (> (printerlist count) 0)
	          (begin (printerlist current 0))
	                 (fill-info (printerlist text))))
	   (lambda(reason) (write-error reason)))))

(vbox layout-policy 100 100
      (hbox
       layout-policy 100 90
       (vbox layout-policy 40 100
             (document:id printerlist (listbox layout-policy 100 90
	                                       header `#(,(_ "Printers"))
                                               rows (map mark-default (get-printers))
                                               (when selected
                                                 ((widgets testpage-button
                                                           default-button
                                                           edit-button
                                                           delete-button) activity #t)
                                                 (fill-info (printerlist text)))))
             (hbox
              (document:id testpage-button (button (_ "Test page")
                                                   inactive
                                                   (when clicked (do-test-page))))
              (document:id default-button (button (_ "Made default")
                                                  inactive
                                                  (when clicked (do-default))))))
       (vbox layout-policy 60 100
             (groupbox (_ "Printer options")
                       layout-policy 100 90
                       (vbox layout-policy 100 -1
                             (hbox layout-policy 100 -1
                                   (label (_ "Name:") layout-policy 20 -1)
                                   (document:id printername (edit ""
                                                                  inactive
                                                                  layout-policy 80 -1
                                                                  (when changed
                                                                   (printerlist row-item
                                                                                (printerlist current)
                                                                                (printername text))))))
                             (document:id connection-button
                                          (button (_ "Connection/Model")
                                                  inactive
                                                  layout-policy 100 -1
                                                  (when clicked (change-connection))))
                             (document:id state-button
                                          (button (_ "stop printer")
                                                               pixmap "printers/pause.png"
                                                               inactive
                                                               (when clicked
                                                                (cell-set!  printer-status
                                                                            (not (cell-ref printer-status)))
                                                                (redraw-status-button))))
                             (document:id error-message (label "" invisible layout-policy 100 -1))))

             (document:id group1 (hbox
                                  layout-policy 100 -1
                                  (document:id edit-button (button (_ "Edit")
                                                                   inactive
                                                                   layout-policy 33 -1
                                                                   (when clicked (do-edit))))
                                  (document:id new-button (button (_ "New")
                                                                  layout-policy 33 -1
                                                                  (when clicked (do-new))))
                                  (document:id delete-button (button (_ "Delete")
                                                                     inactive
                                                                     layout-policy -2 -1
                                                                     (when clicked (do-delete))))))          
             (document:id group2 (hbox layout-policy 100 -1
                                       invisible
                                       (document:id accept-button (button (_ "Accept")
                                                                          layout-policy 50 -1
                                                                          (when clicked
                                                                           (and (modify-printer)
                                                                                (begin
                                                                                  (and (eq? (cell-ref mode) 'new)
                                                                                       (begin
                                                                                         (cell-set! mode 'edit)
                                                                                         (printername inactive)))
                                                                                  (fill-info (printerlist text))
                                                                                  (switch-state))))))
                                       (document:id discard-button (button (_ "Discard")
                                                                           layout-policy 50 -1
                                                                           (when clicked
                                                                            (case (cell-ref mode)
                                                                              ((new)
                                                                               (cell-set! mode 'edit)
                                                                               (printername inactive)
                                                                               (printerlist remove (printerlist current))
                                                                               (and (> (printerlist count) 0) (printerlist current 0))))
                                                                            (and (> (printerlist count) 0)(fill-info (printerlist text)))
                                                                            (switch-state)))))))))

                        
(frame:buttons-view (button (_ "Quit") (when clicked (document:end))) )
